# -*- coding: UTF-8 -*-
"""
load chemical in site database

USAGE

    python -m thyrsis.database.load_chemicals chemicals.csv

    load chemical into the database

    the expected .csv format is:

        - chemicals.csv: "name", "period", "atomic_mass", "radioactive_decay", "specific_activity", "water_quality_limit", "atomic_number", "masses_number"

"""
from builtins import input
from builtins import str

import os
import csv
from ..log import logger
import codecs

def get_float(string):
    """ convert a string into float, even with a ',' decimal limit
    
    :param string: value to convert
    :type string: string

    :return: converted value
    :rtype: float
    """
    return float(string.replace(',','.')) if string != '' else 0.0

def load_chemicals(cur, chemicals):
    """ load chemicals values into the current database
    
    :param cur: cursor on a sqlite connection
    :type cur: sqlite3.Cursor
    :param chemicals: path to a csv file of chemicals
    :type chemicals: string

    :return: None
    """
    nchemicals, = cur.execute("SELECT COUNT(1) from elements_chimiques ").fetchone()
    if nchemicals > 0:
        ans = input(str(nchemicals)+u' chemicals already exist : do you want to remove it ? (yes/no) ')
        if ans[0] == 'y':
            logger.notice(u'Remove chemicals')
            cur.execute("DELETE FROM elements_chimiques")
        else:
            return

    values = []
    with open(chemicals) as chs :
        for i, row in enumerate(csv.reader(chs,delimiter=';')):
            if i == 0 or len(row) != 8:
                continue

            id = i
            nom = str(row[0])
            period = get_float(row[1])
            atomic_mass = get_float(row[2])
            rad_decay = get_float(row[3])
            spe_act = get_float(row[4])
            water_limit = get_float(row[5])
            atomic_nb = int(row[6])
            masses_nb = int(row[7])
            values.append((id,nom, period, atomic_mass, rad_decay, spe_act, water_limit, atomic_nb, masses_nb))

        cur.executemany("""
            INSERT INTO elements_chimiques(id, nom, periode, masse_atomique, decroissance_radioactive, activite_specifique, limite_qualite_eau, numero_atomique, nombre_de_masses)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
            """, values)
        cur.execute("COMMIT")

        nchemicals, = cur.execute("SELECT COUNT(1) from elements_chimiques").fetchone()
        logger.notice(nchemicals, u' chemicals registered')

if __name__ == "__main__":
    import sys
    import sip
    assert(sip.getapi('QVariant') == 2)
    assert(sip.getapi('QString') == 2)
    from PyQt5.QtCore import *
    from PyQt5.QtGui import *
    from ..database import sqlite
    from ..settings import Settings

    logger.enable_console(True)

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) != 2:
        logger.error("wrong number of arguments (try load_chemicals.py --help)")
        exit(1)

    QCoreApplication.setOrganizationName("QGIS")
    QCoreApplication.setOrganizationDomain("qgis.org")
    QCoreApplication.setApplicationName("QGIS2")

    if not Settings().contains("General", "defaultSite"):
        logger.error("defaultSite undefined, use preferences_dialog.py")
        exit(1)

    sitesdb = str(Settings().value("General", "defaultSite"))

    conn = sqlite.connect(sitesdb)

    load_chemicals(conn.cursor(), sys.argv[1])
    conn.commit()
